from Products.CMFCore.CMFCorePermissions import View, AccessContentsInformation, ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseContent, registerType, DisplayList
from AccessControl import ClassSecurityInfo
from OFS.Image import Image
from StringIO import StringIO
from schema import MediaReferenceSchema
from base64 import decodestring
from config import toolId


class MediaReference(BaseContent):
    """ object representing a reference to media on a media server """

    schema = MediaReferenceSchema

    meta_type = 'MediaReference'
    archetype_name = 'Media Reference'
    content_icon = 'media_reference_icon.png'
    immediate_view = 'base_view'
    default_view   = 'base_view'

    actions = ({'id': 'media', 'name': 'Media References',
        'action': 'string:${object_url}/media_references',
        'permissions': (ModifyPortalContent,),
        'visible': False,
    },)

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    security.declareProtected(AccessContentsInformation, 'mediaUID', 'mediaProperties')

    def __init__(self, id, uid, **properties):
        """ initialize the media reference """
	self.id = id
	self.mediaUID = uid
	self.mediaProperties = properties

    security.declareProtected(AccessContentsInformation, 'getProperties')
    def getProperties(self):
        """ return all extra properties saved with the media reference """
	return self.mediaProperties.copy()

    security.declareProtected(AccessContentsInformation, 'displayProperties')
    def displayProperties(self):
        """ return all extra properties saved with the media reference in human readable form """
	out = StringIO()
	for item in self.getProperties().items():
	    print >> out, '%s: %s <br />' % item
	return out.getvalue()

    security.declareProtected(ModifyPortalContent, 'setProperties')
    def setProperties(self, clear=None, **properties):
        """ set extra info for later use of the media reference """
	if clear: self.mediaProperties = properties
	else: self.mediaProperties.update(properties)
	self._p_changed = 1			# notify persistence system

    security.declarePrivate('getMediaTool')
    def getMediaTool(self):
	""" return the media tool instance, which is normally the parent """
        return getToolByName(self, toolId)

    security.declareProtected(AccessContentsInformation, 'getRendererName')
    def getRendererName(self):
        """ return the renderer name from the properties (for convenience) """
	return self.getProperties().get('renderer', None)

    security.declareProtected(AccessContentsInformation, 'getRendererVocabulary')
    def getRendererVocabulary(self):
        """ return ids/names for all render scripts/templates as a vocabulary list """
	content_type = self.getContentType()
	return self.getMediaTool().getRendererVocabulary(content_type)

    security.declareProtected(AccessContentsInformation, 'getPosition')
    def getPosition(self):
        """ return the position value from the properties (for sorting) """
	return self.getProperties().get('position', 0)

    security.declareProtected(AccessContentsInformation, 'getScale')
    def getScale(self):
        """ return the selected media scale from the properties (for convenience) """
	return self.getProperties().get('scale', None)

    security.declareProtected(AccessContentsInformation, 'getMediaScales')
    def getMediaScales(self):
	""" return supported media scales/dimensions """
	return self.getMediaTool().getMediaScales()

    security.declareProtected(AccessContentsInformation, 'getMediaInfo')
    def getMediaInfo(self):
	""" return some details about the media object """
	return self.getMediaTool().getMediaInfo(self.mediaUID)

    security.declareProtected(AccessContentsInformation, 'getContentType')
    def getContentType(self):
        """ return the content-type of the media object via rpc """
	return self.getMediaTool().getContentType(self.mediaUID)

    security.declareProtected(AccessContentsInformation, 'getMediaData')
    def getMediaData(self):
        """ return complete data of the media object via rpc """
	return self.getMediaTool().getMediaData(self.mediaUID)

    security.declareProtected(View, 'render')
    def render(self, REQUEST=None, RESPONSE=None):
        """ render the media object with the currently selected renderer """
	mtool = self.getMediaTool()
	info = mtool.getMediaInfo(self.mediaUID)
        props = self.getProperties()
	renderer = mtool.getRenderer(self.getRendererName())
	renderer = renderer.__of__(self)
	return renderer(REQUEST, RESPONSE, info=info, properties=props)

    security.declarePrivate('getMediaImage')
    def getMediaImage(self, scale=None):
        """ retrieve the scaled image with dimensions provided by the extra properties """
	scale = scale or self.getScale()
        props = self.getProperties()
	return self.getMediaTool().getMediaImage(self.mediaUID, scale, props)

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST, RESPONSE, scale=None):
        """ render the image and return it via its index_html """
    	img = self.getMediaImage(scale)
	RESPONSE.setHeader('Cache-Control', 'no-store no-cache')
	return img.__of__(self).index_html(REQUEST, RESPONSE)

    security.declareProtected(View, 'thumbnail')
    def thumbnail(self, REQUEST, RESPONSE):
        """ return a thumbnail of the media object (for convenience) """
    	img = self.getMediaImage('thumbnail')
	RESPONSE.setHeader('Cache-Control', 'no-store no-cache')
	return img.__of__(self).index_html(REQUEST, RESPONSE)

    security.declareProtected(View, 'download')
    def download(self, REQUEST, RESPONSE):
        """ download the original and complete media data """
        info = self.getMediaInfo()
	set = RESPONSE.setHeader
        set('Content-Disposition', 'attachment; filename="%s"' % info.get('title'))
	set('Cache-Control', 'no-store no-cache')
	set('Content-Type', info.get('content_type'))
        return self.getMediaData()


registerType(MediaReference)
