from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.CMFCorePermissions import View, AccessContentsInformation
from Products.CMFCore.CMFCorePermissions import AddPortalContent, ModifyPortalContent
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.Archetypes.exceptions import ReferenceException
from Products.Archetypes.public import DisplayList
from Products.Archetypes.utils import make_uuid
from MediaReference import MediaReference
from AccessControl import ClassSecurityInfo
from ZPublisher.HTTPRequest import record
from Globals import InitializeClass
from OFS.Folder import Folder
from OFS.Image import Image
from DateTime import DateTime
from xmlrpcauth import authServerProxy
from base64 import decodestring
from config import toolId


defaultMediaScales = (
    { 'name': 'default', 'width': 400, 'height': 400, 'title': 'Default (400x400)' },
    { 'name': 'thumbnail', 'width': 80, 'height': 80, 'title': 'Thumbnail (80x80)' },
)

class NeedHandshake(Exception):
    """ exception for handling handshake between media tool and db """


class MediaTool(UniqueObject, ActionProviderBase, Folder):
    """ tool to communicate with the media db/server """

    id  = toolId
    title = 'Manages RPC to the MediaDB'
    meta_type = 'Media Tool'

    media_db_url = None
    media_db_auth_user = None
    media_db_auth_password = None
    media_scales = ()

    _properties = (
            {'id':'media_db_url', 'type':'string', 'mode':'w'},
            {'id':'media_db_auth_user', 'type':'string', 'mode':'w'},
            {'id':'media_db_auth_password', 'type':'string', 'mode':'w'},
            {'id':'media_scales', 'type':'lines', 'mode':'w'},
    )

    security = ClassSecurityInfo()

#   manage_options = (
#       ({
#           'label': 'Config',
#           'action': 'manage_configForm',
#        },)
#       + Folder.manage_options[:1]
#       + ActionProviderBase.manage_options
#       + Folder.manage_options[2:-1]
#   )

    manage_options = (
          Folder.manage_options[:1]
        + ActionProviderBase.manage_options
        + Folder.manage_options[2:-1]
    )
 
    manage_configForm = PageTemplateFile('www/config', globals())

    def __init__(self):
        """ initialize default settings """
	pass

    security.declarePrivate('getMediaScalesList')
    def getMediaScalesList(self):
        """ return a list of all supported media scales or the defaults """
        scales = []
	infinite = 999999
        for scale in self.getProperty('media_scales'):
            name, width, height, title = scale.split(',', 3)
	    name = name.strip()
	    scales.append({
	        'name': name,
	        'title': title.strip(),
	        'width': int(width.strip() or infinite),
	        'height': int(height.strip() or infinite),
	    })
	return tuple(scales) or defaultMediaScales

    security.declareProtected(AccessContentsInformation, 'getMediaScales')
    def getMediaScales(self):
        """ return a dictionary of all supported media scales """
	scales = {}
	default = None
        for scale in self.getMediaScalesList():
	    name = scale['name']
	    scale['dimensions'] = scale.get('width'), scale.get('height')
	    scale['title'] = scale.get('title', name)
	    scales[name] = scale
	    default = default or scale
        scales['default'] = default
        return scales

    security.declareProtected(AccessContentsInformation, 'getMediaScale')
    def getMediaScale(self, scale=None):
        """ return the dimensions for the given media scale name or the defaults """
        scales = self.getMediaScales()
        return scales.get(scale, scales.get('default'))

    security.declareProtected(AccessContentsInformation, 'getScaleVocabulary')
    def getScaleVocabulary(self):
        """ return ids/names for all scales as a vocabulary list """
	names = []
	for scale in self.getMediaScalesList():
	    title = scale.get('title')
	    if title: names.append((scale.get('name'), title))
        return DisplayList(names)

    security.declarePrivate('getRenderers')
    def getRenderers(self):
        """ collect all render scripts/templates """
	types = ('Filesystem Script (Python)', 'Filesystem Page Template')
        skins = getToolByName(self, 'portal_skins')
	return skins.media_templates.objectItems(types)

    security.declarePrivate('getRenderersFor')
    def getRenderersFor(self, content_type):
        """ return all render scripts/templates suitable for the given content type """
	renderers = []
	for name, renderer in self.getRenderers():
	    try: renders = [ s.strip() for s in renderer.renders.split(',') ]
	    except: continue
	    for type in renders:
	        if type == content_type[:len(type)]:
		    renderers.append(renderer)
	return renderers

    security.declareProtected(AccessContentsInformation, 'getRendererVocabulary')
    def getRendererVocabulary(self, content_type):
        """ return ids/names for all render scripts/templates as a vocabulary list """
	names = []
	for r in self.getRenderersFor(content_type):
	    names.append((r.getId(), r.title_or_id()))
        return DisplayList(names)

    security.declarePrivate('getRenderer')
    def getRenderer(self, id):
        """ return renderer object with the given id (or none) """
	for name, renderer in self.getRenderers():
	    if id == name:
	        return renderer
	return None

    security.declareProtected(AccessContentsInformation, 'getMediaDBUrl')
    def getMediaDBUrl(self):
        """ return the url of the media db/server """
	return self.media_db_url

    security.declarePublic('pingMediaServer')
    def pingMediaServer(self):
        """ test if the media db/server is reachable """
	server = self.rpcServer(timeout=5)
	try: return (server.getId() == 'media_db')
	except: return False

    security.declarePrivate('rpcServer')
    def rpcServer(self, verbose=None, timeout=30):
        """ return a connection object to the media db/server """
	return authServerProxy(self.media_db_url, self.media_db_auth_user,
	    self.media_db_auth_password, verbose, timeout)

    security.declareProtected(AccessContentsInformation, 'getMediaInfo')
    def getMediaInfo(self, uid):
	""" fetch information about the media object from the media db """
	server = self.rpcServer()
        try: info = server.mediaInfo(uid)
        except: info = {}
	return info

    security.declareProtected(AccessContentsInformation, 'getMediaDetails')
    def getMediaDetails(self, uid):
	""" fetch type-specific details about the media object from the media db """
	server = self.rpcServer()
        try: info = server.mediaDetails(uid)
        except: info = {}
	return info

    security.declareProtected(AccessContentsInformation, 'getContentType')
    def getContentType(self, uid):
        """ fetch content type of the media object from the media db """
	server = self.rpcServer()
        try: type = server.mediaContentType(uid)
        except: type = None
	return type

    security.declareProtected(AccessContentsInformation, 'getMediaData')
    def getMediaData(self, uid):
        """ fetch complete data of the media object from the media db """
	server = self.rpcServer()
        try: data = decodestring(server.mediaData(uid))
        except: data = None
	return data

    security.declarePrivate('getMediaImage')
    def getMediaImage(self, uid, scale=None, props={}):
        """ return a scaled version of the media image from the server"""
	server = self.rpcServer()
	width, height = self.getMediaScale(scale).get('dimensions')
        info = server.scaledMediaData(uid, width, height, props)
	title = unicode(info.get('title'), 'utf-8')	# make it unicode first and ...
	title = title.encode('iso-8859-1')		# convert the string to iso 8859
	data = decodestring(info.get('data'))
	return Image('image', title, data)

    security.declareProtected(ModifyPortalContent, 'getThumbnail')
    def getThumbnail(self, REQUEST, RESPONSE, uid, scale='thumbnail'):
        """ return a thumbnail of the media object (for cms previews) """
    	img = self.getMediaImage(uid, scale)
	RESPONSE.setHeader('Cache-Control', 'no-store no-cache')
	return img.__of__(self).index_html(REQUEST, RESPONSE)

    security.declarePublic('getRecordAttributes')
    def getRecordAttributes(self, obj):
        """ return all dictionary values from a http request record class """
	assert isinstance(obj, record)
	return obj.__dict__.copy()

    security.declarePublic('generateID')
    def generateID(self):
        """ generate and return a unique id """
	today = DateTime().strftime('%Y%m%d')
	return '-'.join((today, make_uuid()))

    security.declareProtected(ModifyPortalContent, 'getMediaReference')
    def getMediaReference(self, uid):
        """ look up and return the media reference with the given uid """
	rtool = getToolByName(self, 'reference_catalog')
	obj = rtool.lookupObject(uid)
	if obj is None:
	    raise AttributeError, 'no reference %s' % uid
        assert obj == self._getOb(obj.getId())	# make sure it's a contained media ref
	return obj

    security.declareProtected(ModifyPortalContent, 'getMediaReferences')
    def getMediaReferences(self):
        """ return all existing media references """
	return self.objectValues('MediaReference')

    security.declareProtected(ModifyPortalContent, 'createMediaReference')
    def createMediaReference(self, uid, **properties):
        """ create a media reference refering to the given uid """
        id = self.generateID()
	id = self._setObject(id, MediaReference(id, uid, **properties))
        return self._getOb(id)

    security.declareProtected(ModifyPortalContent, 'destroyMediaReference')
    def destroyMediaReference(self, uid):
        """ destroy a media reference with the given uid """
	obj = self.getMediaReference(uid)
        if obj.getBackReferences():
	    raise ReferenceException, 'Object still has back references.'
	self._delObject(obj.getId())

    security.declareProtected(AccessContentsInformation, 'mediaReferenceCounts')
    def mediaReferenceCounts(self, uids):
        """ return number of usages for the given reference uids """
	counts = dict(map(lambda x: (x, 0), uids))	# initialize the dictionary
	for ref in self.getMediaReferences():
	    if not uids or ref in uids:
	        muid = ref.getMediaUID()
	        try: counts[muid] += 1
	        except: counts[muid] = 1
	return counts

    security.declarePrivate('mediaSessionId')
    def mediaSessionId(self, REQUEST):
        """ retrieves the session id used on the media db server """
	session = REQUEST.SESSION
	msid = session.get('media_session_id', None)
	if not msid:
	    raise NeedHandshake, self.media_db_url	# raise error so nothing gets rendered
	return msid

    security.declareProtected(AccessContentsInformation, 'getMediaCart')
    def getMediaCart(self, REQUEST):
        """ retrieve references stored in media cart on the media db server """
	server = self.rpcServer()
	msid = self.mediaSessionId(REQUEST)
	return server.mediaCartContents(msid)

    security.declareProtected(AccessContentsInformation, 'removeFromMediaCart')
    def removeFromMediaCart(self, REQUEST, references=[]):
        """ removes references from the media cart on the media db server """
	server = self.rpcServer()
	msid = self.mediaSessionId(REQUEST)
	return server.removeFromMediaCart(msid, references)

    security.declareProtected(AccessContentsInformation, 'getFontVocabulary')
    def getFontVocabulary(self):
        """ return ids/names for all fonts as a vocabulary list """
	server = self.rpcServer()
	fonts = server.availableMediaFonts()
        return DisplayList(fonts)

    security.declareProtected(AccessContentsInformation, 'availableMediaInfo')
    def availableMediaInfo(self):
        """ return information about all accessible media objects on the server """
	server = self.rpcServer()
        return server.availableMediaInfo()


InitializeClass(MediaTool)
